import csv
import argparse
import itertools
import logging
from copy import deepcopy

import requests
from . import config
from . import koji_build_msg
from . import koji_tag_msg
from . import TriggerMsgError
from .triggers import get_trigger


trigger = get_trigger(config.trigger_type)

# set up logging
logger = logging.getLogger('Jobrunner')
sh = logging.StreamHandler()
formatter = logging.Formatter(fmt='%(asctime)s - %(name)s - %(levelname)s: %(message)s',
                              datefmt='%Y-%m-%d %H:%M:%S')
sh.setFormatter(formatter)
logger.addHandler(sh)


def trigger_task(item, item_type, task, arch):
    logger.debug('triggering %s for %s on %s' % (task, item, arch))

    output = trigger.trigger_job(item, item_type, task, arch)
    for line in output:
        logger.info(line)


def trigger_tasks(item, item_type, tasks, arches):
    for task, arch in itertools.product(tasks, arches):
        trigger_task(item, item_type, task, arch)


def file_jobs(jobsfile):
    with open(jobsfile, 'r') as csvfile:
        jobs = csv.reader(csvfile, delimiter=';')
        for job in jobs:
            trigger_task(job[1], job[2], job[3], job[4])


def datagrepper_jobs(start, end):
    params = {
        'start': start,
        'end': end,
        'category': 'buildsys',
        'topic': ['org.fedoraproject.prod.buildsys.build.state.change',
                  'org.fedoraproject.prod.buildsys.tag'],
    }

    r = requests.get(config.datagrepper_url, params=params)

    koji_tag_items = set()
    for msg in r.json()['raw_messages']:
        if msg['topic'].endswith('build.state.change'):
            try:
                n, v, r = koji_build_msg.get_nvr(msg)
                item_type = koji_build_msg.ITEM_TYPE
                tasks = config.koji_build_completed_tasks
                trigger_tasks('-'.join((n, v, r)), item_type, tasks, config.valid_arches)
            except TriggerMsgError, e:
                logger.debug(e)
        elif msg['topic'].endswith('tag'):
            try:
                koji_tag_items.add(koji_tag_msg.get_item(msg))
            except TriggerMsgError, e:
                logger.debug(e)
        else:
            logger.warning('Got an unexpected topic %s', msg['topic'])

    item_type = koji_tag_msg.ITEM_TYPE
    for item in koji_tag_items:
        tasks = deepcopy(config.koji_tag_changed_tasks)
        if item.endswith('testing-pending'):
            tasks.remove('upgradepath')
        trigger_tasks(item, item_type, tasks, config.valid_arches)


def main():
    parser = argparse.ArgumentParser(prog='Jobrunner',
                                     usage='Gets jobs from datagrepper from a given time '
                                           'period and runs them.')
    parser.add_argument('-s', '--start', help='Start time (Unix time format)')
    parser.add_argument('-e', '--end', help='End time (Unix time format)')
    parser.add_argument('-d', '--debug', action='store_true', help='Print debug message')
    parser.add_argument('-f', '--jobsfile', help='A file containing jobs in csv format')
    args = parser.parse_args()

    logger.setLevel(logging.DEBUG if args.debug else logging.INFO)

    # let's make running jobs from the file a priority
    if args.jobsfile:
        file_jobs(args.jobsfile)
    else:
        datagrepper_jobs(args.start, args.end)
