import re
from copy import deepcopy

import fedmsg
import fedmsg.encoding

from twisted.internet.task import LoopingCall

from . import config, JobTrigger, TriggerMsgError

ITEM_TYPE = 'koji_tag'


def get_item(msg):
    koji_tag = msg['msg']['tag']
    instance = msg['msg']['instance']

    if not instance == 'primary':
        raise TriggerMsgError('rejecting message not from primary koji instance (%s from %s)'
                              % (koji_tag, instance))

    if not koji_tag.endswith('pending'):
        raise TriggerMsgError('rejecting message for not being a pending tag (%s)' % koji_tag)

    m = re.match(r'^f\d\d-.*', koji_tag)
    if m is None:
        raise TriggerMsgError('rejecting message for not starting with fedora version (e.g. f22) '
                              '(%s)' % koji_tag)

    return koji_tag


class KojiTagChangedJobTrigger(JobTrigger):
    topic = "org.fedoraproject.prod.buildsys.tag"
    jsonify = False
    config_key = 'taskotron.kojitagchangedjobtrigger.enabled'

    def __init__(self, *args, **kw):
        super(KojiTagChangedJobTrigger, self).__init__(*args, **kw)

        self.queued_tags = set()

        # check for bodhi_to_koji_tags tasks queue every config.fuse_delay seconds
        lc = LoopingCall(self.delayed_consume)
        lc.start(config.fuse_delay)

    def delayed_consume(self):
        if self.queued_tags:
            try:
                for tag in self.queued_tags:
                    # XXX ugly: do not schedule upgradepath for updates-testing-pending
                    tasks = deepcopy(config.koji_tag_changed_tasks)
                    if tag.endswith('testing-pending'):
                        tasks.remove('upgradepath')
                    self.trigger_tasks(tag, ITEM_TYPE, tasks, config.valid_arches)
            finally:
                self.queued_tags = set()
        else:
            self.log.debug("Woke up, but there were no messages.")

    def consume(self, message):
        msg = fedmsg.encoding.loads(message.body)

        try:
            item = get_item(msg)

            if not len(self.queued_tags):
                self.log.info('setting fuse (%d seconds) for koji_tag %s', config.fuse_delay, item)
            elif item not in self.queued_tags:
                self.log.info('adding koji_tag %s to the fuse queue', item)

            self.queued_tags.add(item)
        except TriggerMsgError, e:
            self.log.debug(e)
